import pygame, sys, code

from bruce.config import Config
from bruce.baseinterp import BaseInterpreterPage
from bruce.fonts import get_font
from pygame.locals import KEYUP

class Output:
    def __init__(self, method, realstdout):
        self.out = method
        self.realstdout = realstdout
        self.data = ''

    def write(self, data):
        self.data += data
        while '\n' in self.data:
            line, self.data = self.data.split('\n', 1)
            self.out(line)
        #print >>self.realstdout, "outputting %r"%(data)

class MyInterpreter(code.InteractiveInterpreter):
    def __init__(self, locals, method):
        self.outmethod = method
        code.InteractiveInterpreter.__init__(self, locals=locals)

    def execute(self, input):
        old_stdout = sys.stdout
        sys.stdout = Output(self.outmethod, old_stdout)
        more = self.runsource(input)
        sys.stdout = old_stdout
        return more

    def write(self, error):
        while '\n' in error:
            line, error = error.split('\n', 1)
            self.outmethod(line)

FONT_HEIGHT = 24
LINE_HEIGHT = FONT_HEIGHT + 2
fontmono = None
def init(force=False):
    global fontmono, fontBd, em
    if fontmono is not None and not force:
        return
    fontBd = get_font(Config.text_font, FONT_HEIGHT, bold=True)
    fontmono = get_font(Config.code_font, FONT_HEIGHT)
    em = fontmono.size('M')[0]
init()

class InterpreterPage(BaseInterpreterPage):
    '''Run a Python interpreter.

    Backspace deletes text.

    The optional "text" may be used to "auto-type" into the interpreter.
    If activated by control-I any further keystrokes will be taken from
    this text until the end of a line. Hitting return will then invoke
    the line as per usual and move the auto-typer on to the next line.
    Backspace is not handled. Shift-return will complete the current line
    and invoke it.

    The up and down keys will search through the input history.
    '''
    def __init__(self, title='', text='', sysver=False, **kw):
        self.prompt = ">>> "
        self.prompt2 = "... "
        self.sysver = sysver
        super(InterpreterPage, self).__init__(title=title, text=text, **kw)

    def preamble(self):
        if self.sysver:
            for line in sys.version.splitlines():
                self.rendered_lines.append(fontmono.render(line, 1,
                    Config.text_fgcolour))

    def init(self, screen):
        self.interpreter = MyInterpreter({'pygame': pygame}, 
                                         self.renderOutputLine)
        super(InterpreterPage, self).init(screen)

    def processLine(self, input):
        # ignore key-up events (typically the next
        # event in the queue)
        if (input.startswith('pygame.event.wait()') or
                input.startswith('pygame.event.poll()')):
            pygame.event.set_blocked(KEYUP)

        if input == 'help()':
            input = 'print "help() not supported, sorry!"'

        # run the interpreter
        self.captured_input.append(input)
        input = '\n'.join(self.captured_input)
        more = self.interpreter.execute(input)

        if not more:
            self.captured_input = []

        if (input.startswith('pygame.event.wait()') or
                input.startswith('pygame.event.poll()')):
            pygame.event.set_allowed(KEYUP)
        return more





from bruce.page import Page

class SpawnPage(Page):
    def __init__(self, command, pause=True, repeat=False):
        self.run = False
        self.pause = pause
        self.command = command
        self.repeat = repeat

    def init(self, screen):
        pass

    def render(self, screen, deltat):
        from shell import spawner
        # We only allow a command
        if self.repeat or not self.run:
            spawner.spawn(self.command)
            self.run = True
        if not self.pause:
            return True



import popen2, select, os

from pygame import event as Event
from pygame import display as Display
import signal
from pygame import locals as L
try:
    import fcntl
except:
    fcntl = None

# !*($&?*($&
O_NONBLOCK = int('04000', 8)

def killprocs(pids, allpids=None):
    if allpids is None:
        allpids = getPids()
    for pid in pids:
        for child in allpids.get(pid, []):
            killprocs([child], allpids)
        try:
            os.kill(pid, 2)
        except:
            pass

def getPids():
    allpids = {}
    while True:
        try:
            lines = os.popen("ps -eo pid,ppid").readlines()
            break
        except IOError:
            continue
    for line in lines:
        if 'PID' in line: continue
        pid, ppid = line.split()
        allpids.setdefault(int(ppid), []).append(int(pid))
    return allpids

class _Spawner:
    def __init__(self):
        self.pids = set()
        # Setup reaper signal
        signal.signal(signal.SIGCHLD, self.reap)

    def reap(self, *signal):
        (pid, status) = os.waitpid(-1, os.WNOHANG)
        if pid:
            if pid in self.pids:
                self.pids.remove(pid)
            #print "reap: self.pids is", self.pids, "removed", pid
        
    def kill(self, pid=None):
        if pid is not None:
            pids = [pid]
        else:
            pids = self.pids
        for p in list(pids): killprocs([p])
        #print "kill: self.pids is", pids

    def spawn(self, command):
        pid = os.fork()
        if pid != 0:
            # parent
            self.pids.add(pid)
            return
        os.execv("/bin/sh", ["sh", "-c", command])

spawner = _Spawner()

class MyInterpreter:
    error = None
    def __init__(self, outputMethod=None):
        # if outputMethod is None, non-interactive
        self.outputMethod = outputMethod
        if fcntl is None:
            raise RuntimeError("Shell not supported - no fcntl module")

    def runsource(self, command, interactive=True):
        stdout, stdin = popen2.popen4(command)
        flags = fcntl.fcntl (stdout, fcntl.F_GETFL, 0)
        flags = flags | O_NONBLOCK
        fcntl.fcntl (stdout, fcntl.F_SETFL, flags)
        #for i in range(10):
        data = ''
        while True:
            try:
                r, w, e = select.select([stdout,], [], [stdout,], 0.1)
            except select.error:
                r, w, e = None, None, None
            if r:
                ndata = stdout.read()
                if not ndata:
                    break
                data += ndata
                print repr(ndata), repr(data)
                while '\n' in data:
                    line, data = data.split('\n', 1)
                    line = line.rstrip()
                    if self.outputMethod is not None:
                        self.outputMethod(line)
                    else: 
                        print "Shell output:", line
            stop = False
            if self.outputMethod is not None or interactive is False:
                for event in Event.get([L.KEYDOWN,]):
                    if event.type == L.KEYDOWN:
                        if event.key == L.K_c and event.mod & L.KMOD_CTRL:
                            stop = True
                            break
                if stop:
                    killprocs([x.pid for x in popen2._active])
                    break
            Display.flip()

if __name__ == "__main__":
    i = MyInterpreter(None)
    print i.runsource('echo hello ; sleep 2 ; echo hello')






class AutoMixin(object):
    def __init__(self, *args, **kwargs):
        if 'timer' in kwargs:
            self._auto_timer = float(kwargs.get('timer'))
            del kwargs['timer']
        super(AutoMixin, self).__init__(*args, **kwargs)

    def render(self, screen, deltat):
        ret = super(AutoMixin, self).render(screen, deltat)
        if ret: return True
        self._auto_timer -= (deltat/1000.0)
        if self._auto_timer <= 0:
            return True
        



from bruce.config import Config
Config.text_font = 'python_games/Vera.ttf'
Config.bold_font = 'python_games/VeraBd.ttf'
Config.code_font = 'python_games/VeraMono.ttf'
from bruce.main import main
from bruce.pages import *

opening_pages = '''
=Presentations In Python

Using Bruce
the Presentation Tool
---
=How do you make
=a presentation?

Edit some Python code
-Add text, image & code pages
-Create your own page types
-World domination is yours!
'''

pages = []
for page in opening_pages.split('---'):
    pages.append(TextPage(page))

pages.append(InterpreterPage('Zen of Python', 'import this'))

pages.append(ScratchPadInterpreterPage('Draw to the Screen', '''
car = pygame.image.load('python_games/car.png')
car
scratch.blit(car, (100, 100))
rotated = pygame.transform.rotate(car, 45)
scratch.blit(rotated, (150, 150))
'''))

pages.append(CodeFilePage("Sample Presentation", 'example.py'))

pages.extend([
    ImagePage("Sample Image", 'sample.jpg'),
    ImagePage("Sample Image", 'sample.jpg'),
])
pages.append(TextPage('''=Questions?

www.pygame.org
pyopengl.sf.net
home.gna.org/oomadness
www.imitationpickles.org/pgu
'''))

main(pages)



import pygame

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

# TODO use PIL for scaling down - it's so much better at it

class ImagePage(Page):
    def __init__(self, title, image, **kw):
        self.title = title
        self.image = image
        self.border = 0
        super(ImagePage, self).__init__(**kw)


    def init(self, screen):
        # init title

        if self.title:
            FONT_SIZE = int(Config.get(['title_font_size',
                'text_font_size'], 64))
            font = get_font(Config.title_font, FONT_SIZE)
            self.title = font.render(self.title, 1, Config.title_fgcolour)
            self.border = FONT_SIZE

        # load image and resize
        image = pygame.image.load(self.image).convert()
        w = screen.get_rect().width - self.border*2
        h = screen.get_rect().height - self.border*2
        imw, imh = image.get_rect().size
        if imw > w or imh > h:
            if imw > imh:
                ratio = float(w) / imw
                imw = w
                imh *= ratio
            else:
                ratio = float(h) / imh
                imh = h
                imw *= ratio
        self.image = pygame.transform.scale(image, (int(imw), int(imh)))
        super(ImagePage, self).init(screen)

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)

        w = screen.get_rect().width
        if self.title:
            tw = self.title.get_rect().width
            screen.blit(self.title, (w/2-tw/2, 0))

        # border
        w -= self.border*2
        h = screen.get_rect().height

        imw, imh = self.image.get_rect().size
        screen.blit(self.image, (self.border + w/2 - imw/2,
            self.border + h/2 - imh/2))
        super(ImagePage, self).render(screen, deltat)





from pygame import locals as L

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

class TextPage(Page):
    '''Displays some text centered on screen.

    The first line is always bolded.

    Obeys newlines.

    Lines starting withj '-' (dash) are initially hidden and are exposed
    or hidden with the left/right arrow keys. Modified arrow keys are
    ignored here, so will change pages.
    '''
    def __init__(self, text, **kw):
        self.width = 0
        self.height = 0
        self.lines = []
        self.hidden_lines = []
        self.text = text

        self.font_size = int(Config.get('text_font_size', 64))
        self.Fonts = { 
            '=': Config.get(('title_font', 'bold_font'), None),
            '*': Config.get(('bold_font', 'text_font'), None),
            ':': Config.get(('code_font', 'text_font'), None),
        }
        self.Colours = { 
            '=': Config.get(('title_fgcolour', 'bold_fgcolour'), None),
            '*': Config.get(('bold_fgcolour', 'text_fgcolour'), None),
            ':': Config.get(('code_fgcolour', 'text_fgcolour'), None),
        }
        self.bgcolour = Config.get('page_bgcolour')
        # default font, colour
        self.dfont = Config.get('text_font')
        self.dcolour = Config.get('text_fgcolour')
        super(TextPage, self).__init__(**kw)

    def init(self, screen):
        lines = []
        for line in self.text.splitlines():
            if line.startswith('#'): continue

            font = self.dfont
            colour = self.dcolour

            hidden = italic = False
            scale = 1
            while line and line[0] in '<>=:*-/\\':
                ch, line = line[0], line[1:]
                if ch == "\\":
                    break
                elif ch == "-":
                    hidden = True
                elif ch == "<":
                    scale *= 0.75
                elif ch == ">":
                    scale *= 1.33
                elif ch == "/":
                    italic = True
                else:
                    font = self.Fonts.get(ch) or font
                    colour = self.Colours.get(ch) or colour
            font_size = int(self.font_size*scale)
            # XXX support crappy bolding if no bold_font set 
            font = get_font(font, font_size, italic=italic)
            f = font.render(line, 1, colour)
            lines.append((f, hidden))

        # total height
        tHeight = (sum([x.get_size()[1] for x,h in lines]))

        sw, sh = screen.get_size()
        curypos = sh/2 - tHeight / 2
        for line, hidden in lines:
            lw, lh = line.get_size()
            xpos = sw/2 - lw/2
            if hidden:
                self.hidden_lines.append((line, (xpos, curypos)))
            else:
                self.lines.append((line, (xpos, curypos)))
            curypos += lh

        # so we can pop() lines off
        self.hidden_lines.reverse()
        self.popped_lines = []
        super(TextPage, self).init(screen)

    def handleEvent(self, event):
        if (event.type == L.KEYDOWN and 
                event.mod & (L.KMOD_SHIFT|L.KMOD_CTRL|L.KMOD_META|L.KMOD_ALT)):
            return event
        elif ((event.type == L.KEYDOWN and event.key == L.K_RIGHT) or
                    (event.type == L.MOUSEBUTTONDOWN and event.button == 1)):
            if not self.hidden_lines:
                return event
            line = self.hidden_lines.pop()
            self.popped_lines.append(line)
            self.lines.append(line)
            return None
        elif ((event.type == L.KEYDOWN and event.key == L.K_LEFT) or
                    (event.type == L.MOUSEBUTTONDOWN and event.button == 3)):
            if not self.popped_lines:
                return event
            line = self.popped_lines.pop()
            self.lines.pop()
            self.hidden_lines.append(line)
            return None
        return event

    def render(self, screen, deltat):
        screen.fill(self.bgcolour)
        for line, pos in self.lines:
            screen.blit(line, pos)
        super(TextPage, self).render(screen, deltat)




# convenience - list all page types here
from bruce.text import TextPage
from bruce.interp import InterpreterPage
from bruce.scratch_interp import ScratchPadInterpreterPage
from bruce.codefile import CodeFilePage
from bruce.image import ImagePage
from bruce.spawnpage import SpawnPage
from bruce.shellinterp import ShellInterpreterPage
from bruce.mixins import AutoMixin


def reinitPageConfig():
    import bruce.shellinterp
    bruce.shellinterp.init(True)


class AutoImagePage(AutoMixin, ImagePage, object):
    pass
class AutoTextPage(AutoMixin, TextPage, object):
    pass
class AutoCodeFilePage(AutoMixin, CodeFilePage, object):
    pass





import os

path = os.path.abspath(os.path.join(os.getcwd(), __file__))

class Config(object):
    # colour specifications
    page_bgcolour = (0, 0, 0)
    text_fgcolour = (255, 255, 255)
    title_fgcolour = (255, 55, 55)      # kinda red-pink-ish
    code_bgcolour = (20, 20, 20)
    code_fgcolour = (0, 255, 0)
    code_hilitecolour = (255, 255, 255)
    cursor_fgcolour = (200, 200, 255)
    interp_font_size = 24
    codefile_font_size = 24

    # fonts may be a list of system font names or a font file name
    text_font_size = 64
    text_font = 'Vera,Helvetica,Arial'
    title_font = 'Vera,Helvetica,Arial'
    code_font = 'VeraMono,Monaco'       # (Monaco is an Apple font)
    # if not defined, we use the above text_font system font with bold=True
    bold_font = None

    def __init__(self):
        raise RuntimeError("dont instantiate Config, fool!")

    def set(cls, key, val):
        if hasattr(cls, key):
            old = getattr(cls, key)
            if isinstance(old, int):
                val = int(val)
            elif isinstance(old, float):
                val = float(val)
            elif isinstance(old, tuple):
                # a colour
                val = tuple([int(x.strip()) 
                        for x in val.strip('()').split(',')])
            elif isinstance(old, basestring) or old is None:
                pass
            else:
                raise ValueError("dont understand old value type %r"%(old))
        setattr(cls, key, val)
    set = classmethod(set)

    def get(cls, key, dflt=None):
        """ get(key, default)
            key can be a string, or a sequence of strings, in which
            case first one wins. default is returned if none match.
        """
        if isinstance(key, basestring):
            key = [key]
        for k in key:
            if hasattr(cls, k):
                return getattr(cls, k)
            else:
                return dflt
    get = classmethod(get)

    def getfont(cls, key, size, dflt=None):
        """ getfont(key, size, default)
            key can be a string, or a sequence of strings, in which
            case first one wins. default is returned if none match.
            returns a font object or default or None
        """
        from bruce.fonts import get_font
        f = cls.get(key, dflt)
        if f:
            f = get_font(f, size)
        return f
    getfont = classmethod(getfont)
        



import pygame

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

def fit_text((w, h), text):
    # start off trying for best fit
    fontsize = min(Config.codefile_font_size, h / len(text))
    while 1:
        font = get_font(Config.code_font, fontsize)
        tw = th = 0
        for line in text:
            width, height = font.size(line)
            tw = max(width, tw)
            th += height
        if tw < w and th < h:
            return tw, th, fontsize
        fontsize = int(fontsize * .9)

class CodeFilePage(Page):
    def __init__(self, title, file, bgcolour=None, fgcolour=None, 
                    highlight='', skiphashbang=False, **kw):
        self.pagetext = [line.rstrip() for line in open(file)]
        self.title = title
        lines = [line.rstrip() for line in open(file)]

        # handle hidden lines       (end comment #--)
        m = []
        skip = False
        for line in lines:
            if skiphashbang and line.startswith("#!"): continue
            if line.endswith('#--'):
                if not skip:
                    m.append('...')
                    skip = True
                else:
                    continue
            else:
                m.append(line)
                skip = False
        lines = m

        if highlight:
            # handle title
            self.pagetext = []
            for l in lines:
                if not l.startswith('#='):
                    self.pagetext.append(l)
                if l.startswith('#='+highlight):
                    els = l.split(' ', 1)
                    if len(els) == 2:
                        self.title = l[len(highlight)+2:].strip()
        else:
            self.pagetext = [line.rstrip() for line in open(file)]
        self.fgcolour = None
        self.bgcolour = None
        self.highlight = highlight
        super(CodeFilePage, self).__init__(**kw)

    def init(self, screen):
        sw = screen.get_rect().width
        fontsize = 64
        while 1:
            font = get_font(Config.title_font, fontsize)
            if font.size(self.title)[0] < sw:
                break
            fontsize = int(fontsize * .75)
        self.title = font.render(self.title, 1, Config.title_fgcolour)
        th = self.title.get_rect().height

        w, h = screen.get_rect().size
        h -= th
        w, h, fontsize = fit_text((w, h), self.pagetext)
        self.pageblock = pygame.Surface((w + 20, h + 20))
        self.pageblock.fill(self.bgcolour or Config.code_bgcolour)
        font = get_font(Config.code_font, fontsize)
        y = 0
        for line in self.pagetext:
            HL = len(self.highlight)
            colour = self.fgcolour or Config.code_fgcolour
            if self.highlight:
                if line.startswith('#='): continue
                if line.endswith('#'+self.highlight):
                    colour = Config.code_hilitecolour
                if line.endswith('//'+self.highlight):
                    colour = Config.code_hilitecolour
                if len(line) > HL and line[-(HL+1)] == '#':
                    line = line[:-(HL+1)].rstrip()
                if len(line) > HL and line[-(HL+2):-(HL)] == '//':
                    line = line[:-(HL+2)].rstrip()
            line = font.render(line, 1, colour)
            self.pageblock.blit(line, (10, 10+y))
            y += line.get_rect().height
        super(CodeFilePage, self).init(screen)

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)

        sw, sh = screen.get_rect().size

        tw, th = self.title.get_rect().size
        screen.blit(self.title, (sw/2 - tw/2, 0))

        bw, bh = self.pageblock.get_size()
        l = sw/2 - bw/2
        t = th + (sh-th)/2 - bh/2
        screen.blit(self.pageblock, (l, t))
        super(CodeFilePage, self).render(screen, deltat)





from bruce.config import Config
from bruce.fonts import get_font
from bruce.shell import MyInterpreter
from bruce.baseinterp import BaseInterpreterPage

FONT_HEIGHT = 24
LINE_HEIGHT = FONT_HEIGHT + 2

fontmono = None
def init(force=False):
    global fontmono, fontBd, em
    if fontmono is not None and not force:
        return
    fontBd = get_font(Config.text_font, FONT_HEIGHT, bold=True)
    fontmono = get_font(Config.code_font, FONT_HEIGHT)
    em = fontmono.size('M')[0]
init()

class ShellInterpreterPage(BaseInterpreterPage):
    '''Run a "shell" interpreter.

    Backspace deletes text.

    The optional "text" may be used to "auto-type" into the interpreter.
    If activated by control-I any further keystrokes will be taken from
    this text until the end of a line. Hitting return will then invoke
    the line as per usual and move the auto-typer on to the next line.
    Backspace is not handled. Shift-return will complete the current line
    and invoke it.

    The up and down keys will search through the input history.
    '''

    def __init__(self, title='', text='', prompt="% ", **kw):
        self.prompt = prompt
        self.prompt2 = '> '
        super(ShellInterpreterPage, self).__init__(title=title, text=text, 
                                                   **kw)

    def preamble(self): 
        pass

    def init(self, screen):
        self.interpreter = MyInterpreter(self.renderOutputLine)
        super(ShellInterpreterPage, self).init(screen)

    def processLine(self, input):
        more = input.endswith('\\')
        if more:
            self.captured_input.append(input)
        else:
            # run the interpreter
            self.captured_input.append(input)
            input = '\n'.join(self.captured_input)
            # The interpreter does it's own output
            self.interpreter.runsource(input)
        return more






import pygame, sets
from pygame import locals as L

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

FONT_HEIGHT = 24
LINE_HEIGHT = FONT_HEIGHT + 2

fontmono = fontBd = None
def init(force=False):
    global fontmono, fontBd, em
    if fontmono is not None and not force:
        return
    fontBd = get_font(Config.text_font, FONT_HEIGHT, bold=True)
    fontmono = get_font(Config.code_font, FONT_HEIGHT)
    em = fontmono.size('M')[0]
init()


shift_keys = sets.Set((L.K_RSHIFT, L.K_LSHIFT))

class BaseInterpreterPage(Page, object):
    '''Run a "shell" interpreter.

    Backspace deletes text.

    The optional "text" may be used to "auto-type" into the interpreter.
    If activated by control-I any further keystrokes will be taken from
    this text until the end of a line. Hitting return will then invoke
    the line as per usual and move the auto-typer on to the next line.
    Backspace is not handled. Shift-return will complete the current line
    and invoke it.

    The up and down keys will search through the input history.
    '''

    def preamble(self): 
        pass

    def __init__(self, title='', text='', auto_mode=False, **kw):
        cursor = pygame.Surface((10, 2))
        pygame.draw.rect(cursor, Config.cursor_fgcolour, (0, 0, 10, 2))
        self.cursor = cursor
        if title:
            self.title = fontBd.render(title, 1, Config.title_fgcolour)
        else:
            self.title = None

        # figure the text to render and make it poppable
        self.auto_lines = [list(line)
            for line in text.strip().splitlines()]
        self.auto_lines.reverse()
        [line.reverse() for line in self.auto_lines]

        # actual rendered lines to draw to the screen
        self.rendered_lines = []

        # Insert text at the start (optional)
        self.preamble()

        # current typed line
        self.current_line = list(self.prompt)
        line = ''.join(self.current_line)
        self.rendered_lines.append(fontmono.render(line, 1,
                    Config.text_fgcolour))

        # input from the user which doesn't quite form complete
        # interpreter input
        self.captured_input = []

        self.history = []
        self.hist_num = 0

        self.auto_mode = auto_mode
        self.auto_line = []

        self.blink_time = 0
        self.blink_state = False
        super(BaseInterpreterPage, self).__init__(**kw)

    def init(self, screen):
        sh = screen.get_rect().height
        if self.title:
            sh -= self.title.get_rect().height
        self.max_rendered_lines = (sh - LINE_HEIGHT) / LINE_HEIGHT
        sw = screen.get_rect().width
        em = fontmono.size('M')[0]
        self.columns = sw / em
        if sw % em: self.columns -= 1
        if self.auto_mode:
            if self.auto_lines:
                self.auto_line = self.auto_lines.pop()
            else:
                self.auto_mode = False
        super(BaseInterpreterPage, self).init(screen)

    def renderOutputLine(self, line):
        from bruce.main import screen
        for n in range(len(line)/self.columns + 1):
            text = line[n*self.columns:(n+1)*self.columns]
            r = fontmono.render(text, 1, Config.text_fgcolour)
            self.rendered_lines.append(r)
        # possibly remove an extraneous line?
        if len(self.rendered_lines) > self.max_rendered_lines:
            diff = len(self.rendered_lines) - self.max_rendered_lines
            self.rendered_lines = self.rendered_lines[diff:]
        self.render(screen, 1)
        pygame.display.flip()

    def _current_prompt(self):
        if self.captured_input:
            return self.prompt2
        else:
            return self.prompt
    curprompt = property(_current_prompt)

    def handleEvent(self, event):
        turbo_mode = False
        mouse = event.type == L.MOUSEBUTTONDOWN
        key = event.type == L.KEYDOWN
        if mouse or key:
            if key and event.key == L.K_BACKSPACE:
                if not self.auto_mode and (len(self.current_line) > 
                                                    len(self.curprompt)):
                    self.current_line.pop()
                    self.render_current_line()
                return None
            if key and event.key == L.K_u and event.mod & L.KMOD_CTRL:
                self.current_line[:] = list(self.curprompt)
                self.render_current_line()
            if ((key and event.key == L.K_RETURN) or 
                            (mouse and event.button == 2)):
                if key and event.mod & L.KMOD_SHIFT and self.auto_mode:
                    while self.auto_line:
                        glyph = self.auto_line.pop()
                        self.current_line.append(glyph)

                if self.auto_mode and self.auto_line:
                    # don't honor the return key yet
                    return None

                # evaluate the input
                if self.captured_input:
                    # on a second line
                    offset = len(self.prompt2)
                else:
                    offset = len(self.prompt)
                input = ''.join(self.current_line[offset:])

                more = self.processLine(input)

                if not more:
                    self.captured_input = []

                # new line
                if input:
                    self.history.append(self.current_line)
                if more:
                    self.current_line[:] = list(self.prompt2)
                else:
                    self.current_line[:] = list(self.prompt)

                # possibly remove an extraneous line?
                if len(self.rendered_lines) > self.max_rendered_lines:
                    diff = len(self.rendered_lines) - self.max_rendered_lines
                    self.rendered_lines = self.rendered_lines[diff:]

                # render the last line
                line = ''.join(self.current_line)
                r = fontmono.render(line, 1, Config.text_fgcolour)
                self.rendered_lines.append(r)

                self.hist_num = 0
                if self.auto_mode:
                    if not self.auto_lines:
                        self.auto_mode = False
                    else:
                        self.auto_line = self.auto_lines.pop()
                return None
            if key and not event.mod:
                if event.key == L.K_UP and self.history:
                    if not self.hist_num:
                        self.history.append(list(self.current_line))
                        self.hist_num = -1
                    if abs(self.hist_num) < len(self.history):
                        self.hist_num -= 1
                    self.current_line[:] = self.history[self.hist_num]
                    self.render_current_line()
                    return None
                if event.key == L.K_DOWN and self.history:
                    if self.hist_num < -1:
                        self.hist_num += 1
                    self.current_line[:] = self.history[self.hist_num]
                    self.render_current_line()
                    return None
            if key and event.key == L.K_i and event.mod & L.KMOD_CTRL:
                self.auto_mode = not self.auto_mode
                if self.auto_mode:
                    if self.auto_lines:
                        self.auto_line = self.auto_lines.pop()
                    else:
                        self.auto_mode = False
                self.current_line[:] = list(self.prompt)
                return None
            if key and event.key == L.K_d and event.mod & L.KMOD_CTRL:
                # control-D quits interpreter
                return event
            if key and event.key in shift_keys:
                # we use this as a modifier
                return None
            if key and event.mod & (L.KMOD_CTRL|L.KMOD_META|L.KMOD_ALT):
                return event
            if key and event.key == L.K_ESCAPE:
                # for some reason this key has a unicode representation
                return event
            if key and event.key == L.K_TAB:
                turbo_mode = True

            if key:
                glyph = getattr(event, 'unicode', None)

                # is this a useful interpreter key?
                if not glyph or ord(glyph) > 256:
                    return event
            elif mouse:
                if event.button not in (4, 5):
                    return event
                glyph = None

            if turbo_mode:
                while self.auto_line:
                    self.current_line.append(self.auto_line.pop())
                turbo_mode = False
            # use auto key or user key?
            else:
                if self.auto_mode:
                    if self.auto_line:
                        glyph = self.auto_line.pop()
                        if glyph == '':
                            self.current_line.pop()
                            self.render_current_line()
                            return None
                    else:
                        return None
                if glyph:
                    self.current_line.append(glyph)
            self.render_current_line()
            return None

        return event

    def processLine(self, input):
        more = input.endswith('\\')
        if more:
            self.captured_input.append(input)
        else:
            # run the interpreter
            self.captured_input.append(input)
            input = '\n'.join(self.captured_input)
            # The interpreter does it's own output
            self.interpreter.runsource(input)
        return more

    def render_current_line(self):
        line = ''.join(self.current_line)
        self.rendered_lines[-1] = fontmono.render(line, 1, Config.text_fgcolour)

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)

        self.blink_time += deltat
        if self.blink_time > 500:
            self.blink_state = not self.blink_state
            self.blink_time = 0

        toff = 0
        if self.title:
            screen.blit(self.title, (0, 0))
            toff = self.title.get_rect().height

        for n, line in enumerate(self.rendered_lines):
            hoff = toff + n * LINE_HEIGHT
            screen.blit(line, (0, hoff))

        if not self.blink_state:
            x = line.get_rect().width
            pygame.draw.rect(screen, Config.cursor_fgcolour,
                (x, toff + n*LINE_HEIGHT + 32, 10, 2))
        super(BaseInterpreterPage, self).render(screen, deltat)





from time import time
from bruce.fonts import get_font
from bruce.config import Config
import pygame.image

fontCode = get_font(Config.code_font, 30)


class Page(object):
    _last_render_time = None
    _last_render_time_font = None
    _last_render_page = None
    _last_render_page_font = None
    _footer_image = None
    _footer = True
    def __init__(self, **kw):
        if kw.get('nofooter'):
            self._footer = False

    def init(self, screen):
        if self._footer and Config.get('footer_image'):
            image = pygame.image.load(Config.get('footer_image')).convert()
            pos = Config.get('footer_align', 'centre')
            sw = screen.get_rect().width
            sh = screen.get_rect().height
            imw, imh = image.get_rect().size
            ypos = sh - 5 - imh
            if pos in ('centre', 'center'):
                xpos = sw/2 - imw/2
            elif pos == 'left':
                xpos = 5
            else:
                xpos = sw - 5 - imw
            self._footer_image = (image, (xpos,ypos))

    def activate(self, screen):
        pass
    def handleEvent(self, event):
        return event
    def render(self, screen, deltat):
        from bruce.main import options, startTime
        if self._footer and self._footer_image:
            image, pos = self._footer_image
            screen.blit(image, pos)

        if options.timer:
            now = int(time() - startTime)
            if self._last_render_time != now:
                tt = "%02d:%02d"%(now/60, now%60)
                colour = Config.get('clock_colour', (255,255,255))
                self._last_render_time_font = fontCode.render(tt, 1, colour)
                self._last_render_time = now
            h = screen.get_rect().height 
            screen.blit(self._last_render_time_font, (20,h-50))
        if options.pageCount:
            from bruce.main import presentation
            c = presentation.currentPage + 1
            if self._last_render_page != c:
                pp = "% 3d"%(c)
                colour = Config.get('clock_colour', (255,255,255))
                self._last_render_page_font = fontCode.render(pp, 1, colour)
                self._last_render_page = c
            w = screen.get_rect().width 
            h = screen.get_rect().height 
            screen.blit(self._last_render_page_font, (w-80,h-50))




import os

import pygame

fonts = { }

def get_font(name, size, bold=False, italic=False):
    key = (name, bold, italic)
    f = fonts.get(key, {})
    if not size in f:
        if os.path.isfile(name):
            f[size] = pygame.font.Font(name, size)
        else:
            f[size] = pygame.font.SysFont(name, size, bold, italic)
        fonts[key] = f
    return f[size]




import pygame
from pygame import locals as L
from bruce.shell import spawner
from time import time

# Don't init sound!
pygame.display.init()
pygame.font.init()

#from bruce.config import Config

class _options:
    fullscreen = False
    startpage = 1
    timer = False
    pageCount = False

options = _options()

startTime = None
screen = None

def main(pages, fullscreen=False, startpage=1):
    global presentation
    presentation = Presentation(pages)
    presentation.initdisplay(fullscreen)
    try:
        presentation.startdisplay(startpage)
    finally:
        spawner.kill()

class Presentation(object):
    def __init__(self, pages):
        self.pages = pages
        self.currentPage = 0

    def initdisplay(self, fullscreen=False):
        global screen
        # init screen
        viewport = (1024, 768)
        if fullscreen:
            screen_flags = L.HWSURFACE | L.FULLSCREEN | L.DOUBLEBUF
        else:
            screen_flags = L.HWSURFACE | L.DOUBLEBUF
        screen = pygame.display.set_mode(viewport, screen_flags)
        pygame.mouse.set_visible(False)

    def _get_page_position(self):
        return self.currentPage
        
    def _set_page_position(self, new):
        self.currentPage = new
        if self.currentPage >= len(self.pages):
            self.currentPage = len(self.pages)-1
        if self.currentPage < 0:
            self.currentPage = 0

    page_position = property(_get_page_position, _set_page_position)

    def startdisplay(self, startpage):
        global screen, startTime
        if options.timer:
            startTime = time()
        # init the pages
        for page in self.pages:
            page.init(screen)
        # startpage starts at 1.
        self.page_position = int(startpage) - 1
        oldpos = None

        clock = pygame.time.Clock()
        while 1:
            deltat = clock.tick(30)

            # handle events -- NOTE THAT THE PAGE CHANGES AFTER WE
            # HANDLE ALL EVENTS
            for event in pygame.event.get():
                if event.type == L.NOEVENT:
                    continue
                if event.type == L.QUIT:
                    return

                # see if the page will handle the event
                event = page.handleEvent(event)
                if event is None: continue

                if event.type == L.KEYDOWN:
                    if event.key == L.K_ESCAPE:
                        return
                    elif event.key == L.K_f and event.mod & L.KMOD_CTRL:
                        pygame.display.toggle_fullscreen()
                    elif event.key == L.K_p and event.mod & L.KMOD_CTRL:
                        pygame.image.save(screen, '/tmp/screenshot.bmp')
                    elif event.key == L.K_d and event.mod & L.KMOD_CTRL:
                        # == "next page" when in an interpreter
                        self.page_position += 1
                    elif event.key == L.K_c and event.mod & L.KMOD_CTRL:
                        spawner.kill()
                    elif event.key == L.K_RIGHT or event.key == L.K_SPACE:
                        self.page_position += 1
                    elif event.key == L.K_LEFT:
                        self.page_position -= 1
                    elif event.key == L.K_DOWN:
                        self.page_position += 5
                    elif event.key == L.K_UP:
                        self.page_position -= 5
                    elif event.key == L.K_END:
                        self.page_position = len(self.pages)-1
                    elif event.key == L.K_HOME:
                        self.page_position = 0
                elif event.type == L.MOUSEBUTTONDOWN:
                    if event.button == 1:
                        self.page_position += 1
                    elif event.button == 3:
                        self.page_position -= 1

            while True:
                page = self.pages[self.page_position]
                if oldpos != self.page_position:
                    page.activate(screen)
                    oldpos = self.page_position
                # render the current page
                if not page.render(screen, deltat):
                    break
                # render returns 'True' to mean 'autoadvance to next page'
                self.page_position += 1
                pygame.display.flip()
            pygame.display.flip()

